package com.pokkt.thirdparty;

import android.content.Context;
import android.util.Log;
import com.app.pokktsdk.AdNetwork;
import com.app.pokktsdk.PokktConfig;
import com.app.pokktsdk.PokktManager;
import com.app.pokktsdk.delegates.PokktCustomNetworkVideoDelegate;
import com.app.pokktsdk.model.Network;
import com.inmobi.ads.InMobiAdRequestStatus;
import com.inmobi.ads.InMobiInterstitial;
import com.inmobi.sdk.InMobiSdk;

import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

public class InMobiNetwork implements AdNetwork, InMobiInterstitial.InterstitialAdListener {

    private static final String TAG = InMobiNetwork.class.getName();
    private Context context = null;
    private PokktConfig pokktConfig = null;
    private boolean isInitialized = false;
    private Network network = null;
    private InMobiInterstitial interstitial = null;
    private String accountId;
    private long placementId;
    private float amount;
    private static final String ACCOUNT_ID_KEY = "AccountId";
    private static final String PLACEMENT_ID = "PlacementId";
    private static final String CURRENCY_KEY = "coins";

    private boolean isTimedOut = false;
    private boolean isAdAvailableSinceLastCall = false;
    private Timer timer;


    @Override
    public void initNetwork(final Context context, final PokktConfig pokktConfig, final Network network) throws Exception {
        Log.i(TAG, "InMobi init network called!!");
        this.pokktConfig = pokktConfig;

        synchronized (InMobiNetwork.class) {
            if (isInitialized) {
                Log.i(TAG, "InMobi init network already initialized!!");
                return;
            }
            this.context = context;
            this.network = network;
            if (network != null && network.getCustomData() != null && extrasAreValid(network.getCustomData())) {
                accountId = network.getCustomData().get(ACCOUNT_ID_KEY);
                placementId = Long.parseLong(network.getCustomData().get(PLACEMENT_ID));
                amount = Float.parseFloat(network.getCustomData().get(CURRENCY_KEY));
                InMobiSdk.init(context, accountId);
                InMobiSdk.setLogLevel(InMobiSdk.LogLevel.NONE);
                Log.i(TAG, "InMobi init network initialized !!");
                isInitialized = true;
            } else {
                throw new Exception("InMobi Init Configurations Error!");
            }
        }
    }

    private boolean extrasAreValid(Map<String, String> extras) {
        return extras.containsKey(ACCOUNT_ID_KEY)
                && extras.containsKey(PLACEMENT_ID)
                && extras.containsKey(CURRENCY_KEY);
    }

    @Override
    public void cacheVideo(final Network network) throws Exception {
        Log.i(TAG, "InMobi cache video called!!");

        if (isAdAvailableSinceLastCall) {
            isAdAvailableSinceLastCall = false;
            Log.i(TAG, "Video is cached since last call !!");
            PokktCustomNetworkVideoDelegate.onDownloadCompleted(context, pokktConfig, network, amount, "0");
            return;
        }

        try {
            interstitial = new InMobiInterstitial(context, placementId, this);
            interstitial.load();

            //start timer
            startTimer();
        } catch (Exception e) {
            Log.e(TAG,e.getMessage(),e);
            if(timer != null) {
                timer.cancel();
            }
            isAdAvailableSinceLastCall = false;
            PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, "Cache Video Failed", network);
        }
    }

    private void startTimer() {
        isTimedOut = false;
        long duration = PokktManager.getTimeOutDuration(context, network);
        timer = new Timer(true);
        TimerTask timertask = new TimerTask() {
            @Override
            public void run() {
                isAdAvailableSinceLastCall = false;
                isTimedOut = true;
                String reason = "Time Out In Fetching Video";
                Log.e(TAG, reason);
                PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, reason, network);
            }
        };
        timer.schedule(timertask, duration);
    }

    @Override
    public void playVideo() {
        Log.i(TAG, "InMobi play video called!!");
        try {
            if (interstitial.isReady()) {
                interstitial.show();
            } else {
                Log.i(TAG, "No rewarded ad is currently available!!");
                throw new Exception("No Ad Available");
            }
        } catch (Exception e) {
            Log.e(TAG, "Play Video Failed",e);
            PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, true, network.getName());
        }
    }

    @Override
    public void onAdRewardActionCompleted(InMobiInterstitial inMobiInterstitial, Map<Object, Object> map) {
        Log.i(TAG, "InMobi onAdRewardActionCompleted !!");
        PokktCustomNetworkVideoDelegate.onVideoCompleted(context, pokktConfig, network.getName());
        try {
            amount = Float.parseFloat((String) map.get(CURRENCY_KEY));
        } catch (Exception e) {
            Log.e(TAG, "Could not parse currency");
        }
        PokktCustomNetworkVideoDelegate.onVideoGratified(context, pokktConfig, amount, network.getName());
    }

    @Override
    public void onAdDisplayed(InMobiInterstitial inMobiInterstitial) {
        Log.i(TAG, "InMobi onAdDisplayed !");
        PokktCustomNetworkVideoDelegate.onVideoDisplayed(context, pokktConfig, network.getName());
    }

    @Override
    public void onAdDismissed(InMobiInterstitial inMobiInterstitial) {
        Log.i(TAG, "InMobi onAdDismissed !!");
        PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, false, network.getName());
    }

    @Override
    public void onAdInteraction(InMobiInterstitial inMobiInterstitial, Map<Object, Object> map) {
        Log.i(TAG, "InMobi onAdInteraction !!");
    }

    @Override
    public void onAdLoadSucceeded(InMobiInterstitial inMobiInterstitial) {
        Log.i(TAG, "InMobi onAdLoadSucceeded !!");
        if (!isTimedOut) {
            isAdAvailableSinceLastCall = false;
            if (timer != null) {
                timer.cancel();
            }
            PokktCustomNetworkVideoDelegate.onDownloadCompleted(context, pokktConfig, network, amount, "0");
        } else {
            isAdAvailableSinceLastCall = true;
        }

    }

    @Override
    public void onAdLoadFailed(InMobiInterstitial inMobiInterstitial, InMobiAdRequestStatus inMobiAdRequestStatus) {
        Log.i(TAG, "InMobi request status: " + inMobiAdRequestStatus.getMessage() + "code: " + inMobiAdRequestStatus.getStatusCode());
        if (!isTimedOut) {
            isAdAvailableSinceLastCall = false;
            if (timer != null) {
                timer.cancel();
            }
            PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, inMobiAdRequestStatus.getMessage(), network);
        }
    }

    @Override
    public void onUserLeftApplication(InMobiInterstitial inMobiInterstitial) {
        PokktCustomNetworkVideoDelegate.onVideoSkipped(context, pokktConfig, network.getName());
    }
}