package com.pokkt.thirdparty;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.os.Bundle;
import android.util.Log;
import com.app.pokktsdk.AdNetwork;
import com.app.pokktsdk.PokktConfig;
import com.app.pokktsdk.delegates.PokktCustomNetworkVideoDelegate;
import com.app.pokktsdk.model.Network;
import com.supersonic.mediationsdk.logger.SupersonicError;
import com.supersonic.mediationsdk.model.Placement;
import com.supersonic.mediationsdk.sdk.RewardedVideoListener;
import com.supersonic.mediationsdk.sdk.Supersonic;
import com.supersonic.mediationsdk.sdk.SupersonicFactory;

public class SupersonicNetwork implements AdNetwork, Application.ActivityLifecycleCallbacks {

    private String activityClassName = null;
    private Context context = null;
    private PokktConfig pokktConfig = null;

    private static final String TAG = "POKKT_SUPERSONIC";
    private static final String APP_KEY = "appKey";
    private String appKey = "";
    private String userId = "";
    private boolean isInitialized = false;
    private Network network;

    private Supersonic supersonic;

    @Override
    public void initNetwork(Context context, PokktConfig pokktConfig, Network network) throws Exception {
        this.pokktConfig = pokktConfig;

        synchronized (SupersonicNetwork.class) {
            if (isInitialized) {
                Log.d(TAG, "SupersonicNetwork init network already initialized!!");
                return;
            }
            this.context = context;
            this.activityClassName = ((Activity) context).getClass().getName();
            this.network = network;
            ((Activity)context).getApplication().registerActivityLifecycleCallbacks(this);

            if (network != null && network.getCustomData() != null && network.getCustomData().containsKey(APP_KEY)) {
                appKey = network.getCustomData().get(APP_KEY);
            } else {
                throw new Exception("SupersonicNetwork Init Configurations Error!");
            }

            isInitialized = true;
        }

        userId  = pokktConfig.getThirdPartyUserId();

        supersonic = SupersonicFactory.getInstance();

        // register the rewarded video listener and initalize the rewarded video
        supersonic.setRewardedVideoListener(mRewardedVideoListener);

        //Please refer to assets/supersonic.config for initial configurations
        supersonic.initRewardedVideo(((Activity)context), appKey, userId);
    }

    @Override
    public void cacheVideo(final Network network) throws Exception {
        Log.i(TAG,"Cache Video Called");
        if (supersonic.isRewardedVideoAvailable()) {
            PokktCustomNetworkVideoDelegate.onDownloadCompleted(context, pokktConfig, network, 0, "0");
        }else{
            PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, "Supersonic download failed",network);
        }
    }

    @Override
    public void playVideo() {
        Log.i(TAG,"Play Video Called");
        try {
            if (supersonic.isRewardedVideoAvailable())
                supersonic.showRewardedVideo();
        } catch(Exception e) {
            Log.e(TAG, "Play Video Failed",e);
            PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, true, network.getName());
        }
    }

    @Override
    public void onActivityCreated(Activity activity, Bundle bundle) {

    }

    @Override
    public void onActivityStarted(Activity activity) {
    }

    @Override
    public void onActivityResumed(Activity activity) {
        try {
            if (activity.getClass().getName().equals(context.getClass().getName())) {
                Log.d(TAG, "Resumed Supersonic");
                supersonic.onResume(((Activity)context));
            }
        } catch(Exception e) {
            Log.e(TAG, e.getMessage(), e);
        }
    }

    @Override
    public void onActivityPaused(Activity activity) {
        try {
            if (activity.getClass().getName().equals(activityClassName)) {
                Log.d(TAG, "Paused Supersonic");
                supersonic.onPause(activity);
            }
        } catch(Exception e) {
            Log.e(TAG, e.getMessage(), e);
        }
    }

    @Override
    public void onActivityStopped(Activity activity) {
    }

    @Override
    public void onActivitySaveInstanceState(Activity activity, Bundle bundle) {

    }

    @Override
    public void onActivityDestroyed(Activity activity) {
    }

    //Rewarded video listener section
    RewardedVideoListener mRewardedVideoListener = new RewardedVideoListener()
    {

        @Override
        public void onRewardedVideoInitSuccess() {
            // Invoked when initialization of RewardedVideo has finished successfully
            Log.d(TAG, "onRewardedVideoInitSuccess");
        }


        @Override
        public void onRewardedVideoInitFail(SupersonicError supersonicError) {
            Log.d(TAG, "onRewardedVideoInitFail : " + supersonicError.toString() );

            int errorCode =  supersonicError.getErrorCode();
            String errorMessage = supersonicError.getErrorMessage();

            if (errorCode == SupersonicError.ERROR_CODE_GENERIC){
                // do something with a specific error, the complete list of error
                // codes can be found on the documentation page
            }
            PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, supersonicError.toString(),network);
        }

        @Override
        public void onRewardedVideoAdOpened() {
            //Invoked when the RewardedVideo ad view has opened.
            Log.d(TAG, "onRewardedVideoAdOpened");
            PokktCustomNetworkVideoDelegate.onVideoDisplayed(context, pokktConfig, network.getName());
        }

        @Override
        public void onRewardedVideoAdClosed() {
            //Invoked when the user is about to return to the application after closing the rewarded video
            Log.d(TAG, "onRewardedVideoAdClosed");
            PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, false, network.getName());

        }

        @Override
        public void onVideoAvailabilityChanged(boolean available) {
            // ignore
        }

        @Override
        public void onVideoStart() {
            //Invoked when the video ad starts playing.
            Log.d(TAG, "onVideoStart");
            //PokktCustomNetworkVideoDelegate.onVideoDisplayed(context, pokktConfig, network.getName());
        }

        @Override
        public void onVideoEnd() {
            //Invoked when the video ad finishes playing.
            Log.d(TAG, "onVideoEnd");
            PokktCustomNetworkVideoDelegate.onVideoClosed(context, pokktConfig, false, network.getName());
        }

        @Override
        public void onRewardedVideoAdRewarded(Placement placement) {
            // Invoked when the user get rewarded from watching the video.
            // @param - Placement

            // Get the information of the placement
            String rewardName = placement.getRewardName();
            int rewardAmount = placement.getRewardAmount();
            Log.d(TAG, "onRewardedVideoAdRewarded: rewardName=" + rewardName + ", rewardAmount=" + rewardAmount);
            PokktCustomNetworkVideoDelegate.onVideoGratified(context, pokktConfig, rewardAmount, network.getName());
        }

        @Override
        public void onRewardedVideoShowFail(SupersonicError supersonicError) {
            //Toast.makeText(context, supersonicError.getErrorMessage(), Toast.LENGTH_SHORT).show();
            Log.e(TAG, supersonicError.getErrorMessage());
            PokktCustomNetworkVideoDelegate.onDownloadFailed(context, pokktConfig, supersonicError.getErrorMessage(), network);
        }
    };

}
